@@ -15,30 +15,31 @@
 # limitations under the License.
 
 __all__ = [
-    "Neo4JSampledBreadthFirstWalk",
-    "Neo4JDirectedBreadthFirstNeighbors",
+    "Neo4jSampledBreadthFirstWalk",
+    "Neo4jDirectedBreadthFirstNeighbors",
 ]
 
-import numpy as np
-import warnings
-from collections import defaultdict, deque
-
-from ...core.schema import GraphSchema
-from ...core.graph import StellarGraph
-from ...data.explorer import GraphWalk
 from ...core.experimental import experimental
+from .graph import Neo4jStellarGraph, Neo4jStellarDiGraph
 
 
-def _bfs_neighbor_query(sampling_direction):
+def _bfs_neighbor_query(sampling_direction, id_property, node_label=None):
     """
     Generate the Cypher neighbor sampling query for a batch of nodes.
 
     Args:
         sampling_direction (String): indicate type of neighbors needed to sample. Direction must be 'in', 'out' or 'both'.
+        id_property (str): Cypher-escaped property name for node IDs.
+        node_label (str, optional): Common label for all nodes in the graph, if such label exists.
+            Providing this is useful if there are any indexes created on this label (e.g. on node IDs),
+            as it will improve performance of queries.
     Returns:
         The cypher query that samples the neighbor ids for a batch of nodes.
     """
     direction_arrow = {"BOTH": "--", "IN": "<--", "OUT": "-->"}[sampling_direction]
+    match_cur_node = (
+        "MATCH(cur_node)" if node_label is None else f"MATCH(cur_node:{node_label})"
+    )
 
     return f"""
         // expand the list of node id in seperate rows of ids.
@@ -47,13 +48,13 @@ def _bfs_neighbor_query(sampling_direction):
         // for each node id in every row, collect the random list of its neighbors.
         CALL apoc.cypher.run(
 
-            'MATCH(cur_node) WHERE id(cur_node) = $node_id
+            '{match_cur_node} WHERE cur_node.{id_property} = $node_id
 
             // find the neighbors
             MATCH (cur_node){direction_arrow}(neighbors)
 
             // put all ids into a list
-            WITH CASE collect(id(neighbors)) WHEN [] THEN [null] ELSE collect(id(neighbors)) END AS in_neighbors_list
+            WITH CASE collect(neighbors.{id_property}) WHEN [] THEN [null] ELSE collect(neighbors.{id_property}) END AS in_neighbors_list
 
             // pick random nodes with replacement
             WITH apoc.coll.randomItems(in_neighbors_list, $num_samples, True) AS in_samples_list
@@ -66,24 +67,29 @@ def _bfs_neighbor_query(sampling_direction):
 
 
 @experimental(reason="the class is not fully tested")
-class Neo4JSampledBreadthFirstWalk(GraphWalk):
+class Neo4jSampledBreadthFirstWalk:
     """
     Breadth First Walk that generates a sampled number of paths from a starting node.
     It can be used to extract a random sub-graph starting from a set of initial nodes from Neo4j database.
     """
 
-    def run(self, neo4j_graphdb, nodes=None, n=1, n_size=None, seed=None):
+    def __init__(self, graph):
+        if not isinstance(graph, Neo4jStellarGraph):
+            raise TypeError("Graph must be a Neo4jStellarGraph or Neo4jStellarDiGraph.")
+        self.graph = graph
+
+    def run(self, nodes=None, n=1, n_size=None):
         """
-        Send queries to Neo4j graph databases and collect sampled breadth-first walks starting from the root nodes.
+        Send queries to Neo4j graph databases and collect sampled breadth-first walks starting from
+        the root nodes.
 
         Args:
-            neo4j_graphdb: (py2neo.Graph) the Neo4j Graph Database object
-            nodes (list): A list of root node ids such that from each node n BFWs will be generated up to the
-            given depth d.
-            n_size (int): The number of neighbouring nodes to expand at each depth of the walk. Sampling of
-            n (int, default 1): Number of walks per node id.
-            neighbours with replacement is always used regardless of the node degree and number of neighbours
-            requested.
+            nodes (list of hashable): A list of root node ids such that from each node n BFWs will
+                be generated up to the given depth d.
+            n_size (list of int): The number of neighbouring nodes to expand at each depth of the
+                walk. Sampling of neighbours with replacement is always used regardless of the node
+                degree and number of neighbours requested.
+            n (int): Number of walks per node id.
             seed (int, optional): Random number generator seed; default is None
 
         Returns:
@@ -91,12 +97,16 @@ class Neo4JSampledBreadthFirstWalk(GraphWalk):
         """
 
         samples = [[head_node for head_node in nodes for _ in range(n)]]
-        neighbor_query = _bfs_neighbor_query(sampling_direction="BOTH")
+        neighbor_query = _bfs_neighbor_query(
+            sampling_direction="BOTH",
+            id_property=self.graph.cypher_id_property,
+            node_label=self.graph.cypher_node_label,
+        )
 
         # this sends O(number of hops) queries to the database, because the code is cleanest like that
         for num_sample in n_size:
             cur_nodes = samples[-1]
-            result = neo4j_graphdb.run(
+            result = self.graph.graph_db.run(
                 neighbor_query,
                 parameters={"node_id_list": cur_nodes, "num_samples": num_sample},
             )
@@ -106,26 +116,27 @@ class Neo4JSampledBreadthFirstWalk(GraphWalk):
 
 
 @experimental(reason="the class is not fully tested")
-class Neo4JDirectedBreadthFirstNeighbors(GraphWalk):
+class Neo4jDirectedBreadthFirstNeighbors:
     """
     Breadth First Walk that generates a sampled number of paths from a starting node.
     It can be used to extract a random sub-graph starting from a set of initial nodes from Neo4j database.
     """
 
-    def run(
-        self, neo4j_graphdb, nodes=None, n=1, in_size=None, out_size=None, seed=None
-    ):
+    def __init__(self, graph):
+        if not isinstance(graph, Neo4jStellarDiGraph):
+            raise TypeError("Graph must be a Neo4jStellarDiGraph.")
+        self.graph = graph
+
+    def run(self, nodes=None, n=1, in_size=None, out_size=None):
         """
         Send queries to Neo4j databases and collect sampled breadth-first walks starting from the root nodes.
 
         Args:
-            neo4j_graphdb (py2neo.Graph): the Neo4j Graph Database object
-            nodes (list): A list of root node ids such that from each node n BFWs will be generated up to the
-            given depth d.
+            nodes (list of hashable): A list of root node ids such that from each node n BFWs will
+                be generated up to the given depth d.
             n (int): Number of walks per node id.
-            in_size (list): The number of in-directed nodes to sample with replacement at each depth of the walk.
-            out_size (list): The number of out-directed nodes to sample with replacement at each depth of the walk.
-            seed (int): Random number generator seed; default is None
+            in_size (list of int): The number of in-directed nodes to sample with replacement at each depth of the walk.
+            out_size (list of int): The number of out-directed nodes to sample with replacement at each depth of the walk.
         Returns:
             A list of multi-hop neighbourhood samples. Each sample expresses a collection of nodes, which could be either in-neighbors,
             or out-neighbors of the previous hops.
@@ -137,15 +148,21 @@ class Neo4JDirectedBreadthFirstNeighbors(GraphWalk):
             ...
             ]
         """
-
-        self._check_neighbourhood_sizes(in_size, out_size)
-        self._check_common_parameters(nodes, n, len(in_size), seed)
+        # FIXME: we may want to run validation on all the run parameters similar to other GraphWalk classes
 
         head_nodes = [head_node for head_node in nodes for _ in range(n)]
         hops = [[head_nodes]]
 
-        in_sample_query = _bfs_neighbor_query(sampling_direction="IN")
-        out_sample_query = _bfs_neighbor_query(sampling_direction="OUT")
+        in_sample_query = _bfs_neighbor_query(
+            sampling_direction="IN",
+            id_property=self.graph.cypher_id_property,
+            node_label=self.graph.cypher_node_label,
+        )
+        out_sample_query = _bfs_neighbor_query(
+            sampling_direction="OUT",
+            id_property=self.graph.cypher_id_property,
+            node_label=self.graph.cypher_node_label,
+        )
 
         # this sends O(2^number of hops) queries to the database, because the code is cleanest like that
         for in_num, out_num in zip(in_size, out_size):
@@ -153,14 +170,14 @@ class Neo4JDirectedBreadthFirstNeighbors(GraphWalk):
             this_hop = []
             for cur_nodes in last_hop:
                 # get in-neighbor nodes
-                neighbor_records = neo4j_graphdb.run(
+                neighbor_records = self.graph.graph_db.run(
                     in_sample_query,
                     parameters={"node_id_list": cur_nodes, "num_samples": in_num},
                 )
                 this_hop.append(neighbor_records.data()[0]["next_samples"])
 
                 # get out-neighbor nodes
-                neighbor_records = neo4j_graphdb.run(
+                neighbor_records = self.graph.graph_db.run(
                     out_sample_query,
                     parameters={"node_id_list": cur_nodes, "num_samples": out_num},
                 )
@@ -169,19 +186,3 @@ class Neo4JDirectedBreadthFirstNeighbors(GraphWalk):
             hops.append(this_hop)
 
         return sum(hops, [])
-
-    def _check_neighbourhood_sizes(self, in_size, out_size):
-        """
-        Checks that the parameter values are valid or raises ValueError exceptions with a message indicating the
-        parameter (the first one encountered in the checks) with invalid value.
-
-        Args:
-            in_size (list): The number of in-directed nodes to sample with replacement at each depth of the walk.
-            out_size (list): The number of out-directed nodes to sample with replacement at each depth of the walk.
-        """
-        self._check_sizes(in_size)
-        self._check_sizes(out_size)
-        if len(in_size) != len(out_size):
-            self._raise_error(
-                "The number of hops for the in and out neighbourhoods must be the same."
-            )
