@@ -37,10 +37,10 @@ class EdgeSplitter(object):
     can be used and it must be in the format dd/mm/yyyy. A date to be used as a threshold value such that only
     edges that have date after the threshold must be given. This effects only the sampling of positive edges.
 
-    Negative edges are sampled at random by uniformly (for 'global' method) selecting two nodes in the graph and
+    Negative edges are sampled at random by (for 'global' method) selecting two nodes in the graph and
     then checking if these edges are connected or not. If not, the pair of nodes is considered a negative sample.
     Otherwise, it is discarded and the process repeats. Alternatively, negative edges are sampled (for 'local' method)
-    using DFS search at a distance from the source node (selected uniformly at random from all nodes in the graph)
+    using DFS search at a distance from the source node (selected at random from all nodes in the graph)
     sampled according to a given set of probabilities.
 
     Positive edges can be sampled so that when they are subsequently removed from the graph, the reduced graph is either
@@ -299,8 +299,8 @@ class EdgeSplitter(object):
              in the original graph. If the graph is heterogeneous, the percentage is calculated
              as a function of the total number of edges that satisfy the edge_label, edge_attribute_label and
              edge_attribute_threshold values given.
-            method (str): How negative edges are sampled. If 'global', then nodes are selected uniformly at random.
-             If 'local' then the first nodes is sampled uniformly from all nodes in the graph, but the second node is
+            method (str): How negative edges are sampled. If 'global', then nodes are selected at random.
+             If 'local' then the first nodes is sampled from all nodes in the graph, but the second node is
              chosen to be from the former's local neighbourhood.
             probs (list): list The probabilities for sampling a node that is k-hops from the source node,
              e.g., [0.25, 0.75] means that there is a 0.25 probability that the target node will be 1-hope away from the
@@ -538,8 +538,6 @@ class EdgeSplitter(object):
                 self.g_train.remove_edge(*edge)
 
                 count += 1
-                if count % 1000 == 0:
-                    print("Removed", count, "edges")
             if count == num_edges_to_remove:
                 return removed_edges
 
@@ -593,8 +591,6 @@ class EdgeSplitter(object):
                 )  # the last entry is the label
                 self.g_train.remove_edge(*edge)
                 count += 1
-                if count % 1000 == 0:
-                    print("Removed", count, "edges")
             if count == num_edges_to_remove:
                 return removed_edges
 
@@ -657,8 +653,6 @@ class EdgeSplitter(object):
                 self.g_train.remove_edge(*edge)
 
                 count += 1
-                if count % 1000 == 0:
-                    print("Removed", count, "edges")
             if count == num_edges_to_remove:
                 return removed_edges
 
@@ -785,8 +779,6 @@ class EdgeSplitter(object):
                                 sampled_edges_set.add((u[0], v))
                                 sampled_edges_set.add((v, u[0]))
                                 count += 1
-                                if count % 1000 == 0:
-                                    print("Sampled {} negatives".format(count))
                                 self.negative_edge_node_distances.append(d)
                             break
                         elif dv < d:
@@ -901,8 +893,6 @@ class EdgeSplitter(object):
                                 sampled_edges_set.add((v, u))
                                 count += 1
                                 self.negative_edge_node_distances.append(d)
-                                if count % 1000 == 0:
-                                    print("Sampled {} negatives".format(count))
                                 break
                         elif dv < d:
                             neighbours = list(nx.neighbors(self.g, v))
@@ -974,8 +964,6 @@ class EdgeSplitter(object):
                     count += 1
                 if count == num_edges_to_sample:
                     return sampled_edges
-                if count % 1000 == 0:
-                    print("Sampled {} negative examples".format(count))
 
         if len(sampled_edges) != num_edges_to_sample:
             raise ValueError(
@@ -1053,8 +1041,6 @@ class EdgeSplitter(object):
                     )  # the last entry is the class label
                     sampled_edges_set.update({(u[0], v[0]), (v[0], u[0])})
                     count += 1
-                    if count % 1000 == 0:
-                        print("Sampled", count, "negative edges")
 
                     if count == num_edges_to_sample:
                         return sampled_edges
