@@ -160,7 +160,7 @@ class GraphWalk(object):
         return rs
 
     def neighbors(self, node):
-        return self.graph.neighbors(node, use_ilocs=True)
+        return self.graph.neighbor_arrays(node, use_ilocs=True)
 
     def run(self, *args, **kwargs):
         """
@@ -280,8 +280,8 @@ class UniformRandomWalk(RandomWalk):
         walk = [start_node]
         current_node = start_node
         for _ in range(length - 1):
-            neighbours = self.graph.neighbors(current_node, use_ilocs=True)
-            if not neighbours:
+            neighbours = self.graph.neighbor_arrays(current_node, use_ilocs=True)
+            if len(neighbours) == 0:
                 # dead end, so stop
                 break
             else:
@@ -303,26 +303,8 @@ def naive_weighted_choices(rs, weights):
     requires a lot of preprocessing (normalized probabilties), and
     does a lot of conversions/checks/preprocessing internally.
     """
-
-    # divide the interval [0, sum(weights)) into len(weights)
-    # subintervals [x_i, x_{i+1}), where the width x_{i+1} - x_i ==
-    # weights[i]
-    subinterval_ends = []
-    running_total = 0
-    for w in weights:
-        if w < 0:
-            raise ValueError("Detected negative weight: {}".format(w))
-        running_total += w
-        subinterval_ends.append(running_total)
-
-    # pick a place in the overall interval
-    x = rs.random() * running_total
-
-    # find the subinterval that contains the place, by looking for the
-    # first subinterval where the end is (strictly) after it
-    for idx, end in enumerate(subinterval_ends):
-        if x < end:
-            break
+    probs = np.cumsum(weights)
+    idx = np.searchsorted(probs, rs.random() * probs[-1], side="left")
 
     return idx
 
@@ -364,12 +346,15 @@ class BiasedRandomWalk(RandomWalk):
             return
 
         # Check that all edge weights are greater than or equal to 0.
-        edges, weights = self.graph.edges(include_edge_weight=True, use_ilocs=True)
+        source, target, _, weights = self.graph.edge_arrays(
+            include_edge_weight=True, use_ilocs=True
+        )
         (invalid,) = np.where((weights < 0) | ~np.isfinite(weights))
         if len(invalid) > 0:
 
             def format(idx):
-                s, t = edges[idx]
+                s = source[idx]
+                t = target[idx]
                 w = weights[idx]
                 return f"{s!r} to {t!r} (weight = {w})"
 
@@ -414,30 +399,19 @@ class BiasedRandomWalk(RandomWalk):
         if weighted:
             self._check_weights_valid()
 
-            # calculate the appropriate unnormalised transition probability, given the history of
-            # the walk
-            def transition_probability(nn):
-                nn, weight = nn
+        weight_dtype = self.graph._edges.weights.dtype
+        cast_func = np.cast[weight_dtype]
+        ip = cast_func(1.0 / p)
+        iq = cast_func(1.0 / q)
 
-                if nn == previous_node:  # d_tx = 0
-                    return ip * weight
-                elif any(nn == pn for pn, _ in previous_node_neighbours):  # d_tx = 1
-                    return weight
-                else:  # d_tx = 2
-                    return iq * weight
-
-        else:
-            # without weights
-            def transition_probability(nn):
-                if nn == previous_node:  ## d_tx = 0
-                    return ip
-                elif nn in previous_node_neighbours:  # d_tx = 1
-                    return 1.0
-                else:  # d_tx = 2
-                    return iq
-
-        ip = 1.0 / p
-        iq = 1.0 / q
+        if np.isinf(ip):
+            raise ValueError(
+                f"p: value ({p}) is too small. It must be possible to represent 1/p in {weight_dtype}, but this value overflows to infinity."
+            )
+        if np.isinf(iq):
+            raise ValueError(
+                f"q: value ({q}) is too small. It must be possible to represent 1/q in {weight_dtype}, but this value overflows to infinity."
+            )
 
         walks = []
         for node in nodes:  # iterate over root nodes
@@ -451,24 +425,30 @@ class BiasedRandomWalk(RandomWalk):
                 current_node = node
 
                 for _ in range(length - 1):
-                    neighbours = self.graph.neighbors(
-                        current_node, use_ilocs=True, include_edge_weight=weighted
-                    )
-
-                    if not neighbours:
-                        break
-
                     # select one of the neighbours using the
                     # appropriate transition probabilities
-                    choice = naive_weighted_choices(
-                        rs, (transition_probability(nn) for nn in neighbours),
-                    )
+                    if weighted:
+                        neighbours, weights = self.graph.neighbor_arrays(
+                            current_node, include_edge_weight=True, use_ilocs=True
+                        )
+                    else:
+                        neighbours = self.graph.neighbor_arrays(
+                            current_node, use_ilocs=True
+                        )
+                        weights = np.ones(neighbours.shape, dtype=weight_dtype)
+                    if len(neighbours) == 0:
+                        break
+
+                    mask = neighbours == previous_node
+                    weights[mask] *= ip
+                    mask |= np.isin(neighbours, previous_node_neighbours)
+                    weights[~mask] *= iq
+
+                    choice = naive_weighted_choices(rs, weights)
 
                     previous_node = current_node
                     previous_node_neighbours = neighbours
                     current_node = neighbours[choice]
-                    if weighted:
-                        current_node = current_node.node
 
                     walk.append(current_node)
 
@@ -572,14 +552,19 @@ class UniformRandomMetaPathWalk(RandomWalk):
                     for d in range(length):
                         walk.append(current_node)
                         # d+1 can also be used to index metapath to retrieve the node type for the next step in the walk
-                        neighbours = self.graph.neighbors(current_node, use_ilocs=True)
+                        neighbours = self.graph.neighbor_arrays(
+                            current_node, use_ilocs=True
+                        )
                         # filter these by node type
+                        neighbour_types = self.graph.node_type(
+                            neighbours, use_ilocs=True
+                        )
                         neighbours = [
-                            n_node
-                            for n_node in neighbours
-                            if self.graph.node_type(n_node, use_ilocs=True)
-                            == metapath[d]
+                            neigh
+                            for neigh, neigh_type in zip(neighbours, neighbour_types)
+                            if neigh_type == metapath[d]
                         ]
+
                         if len(neighbours) == 0:
                             # if no neighbours of the required type as dictated by the metapath exist, then stop.
                             break
@@ -674,7 +659,9 @@ class SampledBreadthFirstWalk(GraphWalk):
                         continue
 
                     neighbours = (
-                        self.neighbors(cur_node) if cur_node is not None else []
+                        self.graph.neighbor_arrays(cur_node, use_ilocs=True)
+                        if cur_node != -1
+                        else []
                     )
                     if len(neighbours) == 0:
                         # Either node is unconnected or is in directed graph with no out-nodes.
@@ -887,14 +874,14 @@ class DirectedBreadthFirstNeighbours(GraphWalk):
             The fixed-length list of neighbouring nodes (or None values
             if the neighbourhood is empty).
         """
-        if node is None:
+        if node == -1:
             # Non-node, e.g. previously sampled from empty neighbourhood
             return [-1] * size
-        neighbours = list(
-            self.graph.in_nodes(node, use_ilocs=True)
-            if idx == 0
-            else self.graph.out_nodes(node, use_ilocs=True)
-        )
+
+        if idx == 0:
+            neighbours = self.graph.in_node_arrays(node, use_ilocs=True)
+        else:
+            neighbours = self.graph.out_node_arrays(node, use_ilocs=True)
         if len(neighbours) == 0:
             # Sampling from empty neighbourhood
             return [-1] * size
@@ -1051,7 +1038,7 @@ class TemporalRandomWalk(GraphWalk):
         walks = []
         num_cw_curr = 0
 
-        edges, times = self.graph.edges(include_edge_weight=True)
+        sources, targets, _, times = self.graph.edge_arrays(include_edge_weight=True)
         edge_biases = self._temporal_biases(
             times, None, bias_type=initial_edge_bias, is_forward=False,
         )
@@ -1068,8 +1055,9 @@ class TemporalRandomWalk(GraphWalk):
 
         # loop runs until we have enough context windows in total
         while num_cw_curr < num_cw:
-            first_edge_index = self._sample(len(edges), edge_biases, np_rs)
-            src, dst = edges[first_edge_index]
+            first_edge_index = self._sample(len(times), edge_biases, np_rs)
+            src = sources[first_edge_index]
+            dst = targets[first_edge_index]
             t = times[first_edge_index]
 
             remaining_length = num_cw - num_cw_curr + cw_size - 1
@@ -1122,18 +1110,15 @@ class TemporalRandomWalk(GraphWalk):
         Perform 1 temporal step from a node. Returns None if a dead-end is reached.
 
         """
+        neighbours, times = self.graph.neighbor_arrays(node, include_edge_weight=True)
+        neighbours = neighbours[times > time]
+        times = times[times > time]
 
-        neighbours = [
-            (neighbour, t)
-            for neighbour, t in self.graph.neighbors(node, include_edge_weight=True)
-            if t > time
-        ]
-
-        if neighbours:
-            times = [t for _, t in neighbours]
+        if len(neighbours) > 0:
             biases = self._temporal_biases(times, time, bias_type, is_forward=True)
             chosen_neighbour_index = self._sample(len(neighbours), biases, np_rs)
-            next_node, next_time = neighbours[chosen_neighbour_index]
+            next_node = neighbours[chosen_neighbour_index]
+            next_time = times[chosen_neighbour_index]
             return next_node, next_time
         else:
             return None
