@@ -529,6 +529,179 @@ class DistMult:
     build = deprecated_model_function(in_out_tensors, "build")
 
 
+class RotatEScore(Layer):
+    def __init__(self, margin, norm_order, **kwargs):
+        super().__init__(**kwargs)
+        self._margin = margin
+        self._norm_order = norm_order
+
+    def get_config(self):
+        return {
+            **super().get_config(),
+            "margin": self._margin,
+            "norm_order": self._norm_order,
+        }
+
+    def call(self, inputs):
+        s_re, s_im, r_re, r_im, o_re, o_im = inputs
+        # expansion of s◦r - t
+        re = s_re * r_re - s_im * r_im - o_re
+        im = s_re * r_im + s_im * r_re - o_im
+        # norm the vector: -|| ... ||_p
+        return self._margin - tf.norm(
+            tf.sqrt(re * re + im * im), ord=self._norm_order, axis=2
+        )
+
+
+@experimental(reason="demo and documentation is missing", issues=[1549, 1550])
+class RotatE:
+    """
+    Implementation of https://arxiv.org/abs/1902.10197
+    """
+
+    def __init__(
+        self,
+        generator,
+        embedding_dimension,
+        # default taken from the paper's code: https://github.com/DeepGraphLearning/KnowledgeGraphEmbedding
+        margin=12.0,
+        # default taken from the paper's code: https://github.com/DeepGraphLearning/KnowledgeGraphEmbedding
+        norm_order=2,
+        embeddings_initializer="normal",
+        embeddings_regularizer=None,
+    ):
+        if not isinstance(generator, KGTripleGenerator):
+            raise TypeError(
+                f"generator: expected KGTripleGenerator, found {type(generator).__name__}"
+            )
+
+        graph = generator.G
+        self.num_nodes = graph.number_of_nodes()
+        self.num_edge_types = len(graph._edges.types)
+        self.embedding_dimension = embedding_dimension
+
+        self._scoring = RotatEScore(margin=margin, norm_order=norm_order)
+
+        def embed(count):
+            return Embedding(
+                count,
+                embedding_dimension,
+                embeddings_initializer=embeddings_initializer,
+                embeddings_regularizer=embeddings_regularizer,
+            )
+
+        # RotatE generates embeddings in C, which we model as separate real and imaginary embeddings
+        # for node types, and just the phase for edge types (since they have |x| = 1)
+        self._node_embeddings_real = embed(self.num_nodes)
+        self._node_embeddings_imag = embed(self.num_nodes)
+
+        # it doesn't make sense to regularize the phase, because it's circular
+        self._edge_type_embeddings_phase = Embedding(
+            self.num_edge_types,
+            embedding_dimension,
+            embeddings_initializer=embeddings_initializer,
+        )
+
+    def embeddings(self):
+        node = 1j * self._node_embeddings_imag.embeddings.numpy()
+        node += self._node_embeddings_real.embeddings.numpy()
+
+        phase = self._edge_type_embeddings_phase.embeddings.numpy()
+        rel = 1j * np.sin(phase)
+        rel += np.cos(phase)
+
+        return node, rel
+
+    def rank_edges_against_all_nodes(
+        self, test_data, known_edges_graph, tie_breaking="random"
+    ):
+        if not isinstance(test_data, KGTripleSequence):
+            raise TypeError(
+                "test_data: expected KGTripleSequence; found {type(test_data).__name__}"
+            )
+
+        num_nodes = known_edges_graph.number_of_nodes()
+
+        all_node_embs, all_rel_embs = self.embeddings()
+
+        raws = []
+        filtereds = []
+
+        # run through the batches and compute the ranks for each one
+        num_tested = 0
+        for ((subjects, rels, objects),) in test_data:
+            num_tested += len(subjects)
+
+            # batch_size x k
+            ss = all_node_embs[subjects, :]
+            rs = all_rel_embs[rels, :]
+            os = all_node_embs[objects, :]
+
+            # (the margin is a fixed offset that doesn't affect relative ranks)
+            mod_o_pred = -np.linalg.norm(
+                (ss * rs)[None, :, :] - all_node_embs[:, None, :], axis=2
+            )
+            mod_s_pred = -np.linalg.norm(
+                (all_node_embs)[:, None, :] * rs[None, :, :] - os[None, :, :], axis=2
+            )
+
+            mod_o_raw, mod_o_filt = _ranks_from_score_columns(
+                mod_o_pred,
+                true_modified_node_ilocs=objects,
+                unmodified_node_ilocs=subjects,
+                true_rel_ilocs=rels,
+                modified_object=True,
+                known_edges_graph=known_edges_graph,
+                tie_breaking=tie_breaking,
+            )
+            mod_s_raw, mod_s_filt = _ranks_from_score_columns(
+                mod_s_pred,
+                true_modified_node_ilocs=subjects,
+                true_rel_ilocs=rels,
+                modified_object=False,
+                unmodified_node_ilocs=objects,
+                known_edges_graph=known_edges_graph,
+                tie_breaking=tie_breaking,
+            )
+
+            raws.append(np.column_stack((mod_o_raw, mod_s_raw)))
+            filtereds.append(np.column_stack((mod_o_filt, mod_s_filt)))
+
+        # make one big array
+        raw = np.concatenate(raws)
+        filtered = np.concatenate(filtereds)
+        # for each edge, there should be an pair of raw ranks
+        assert raw.shape == filtered.shape == (num_tested, 2)
+
+        return raw, filtered
+
+    def __call__(self, x):
+        s_iloc, r_iloc, o_iloc = x
+
+        s_re = self._node_embeddings_real(s_iloc)
+        s_im = self._node_embeddings_imag(s_iloc)
+
+        r_phase = self._edge_type_embeddings_phase(r_iloc)
+
+        r_re = tf.math.cos(r_phase)
+        r_im = tf.math.sin(r_phase)
+
+        o_re = self._node_embeddings_real(o_iloc)
+        o_im = self._node_embeddings_imag(o_iloc)
+
+        return self._scoring([s_re, s_im, r_re, r_im, o_re, o_im])
+
+    def in_out_tensors(self):
+        s_iloc = Input(shape=1)
+        r_iloc = Input(shape=1)
+        o_iloc = Input(shape=1)
+
+        x_inp = [s_iloc, r_iloc, o_iloc]
+        x_out = self(x_inp)
+
+        return x_inp, x_out
+
+
 def _ranks_from_comparisons(greater, greater_equal, tie_breaking):
     strict = 1 + greater.sum(axis=0)
     # with_ties - strict = the number of elements exactly equal (including the true edge itself)
@@ -602,27 +775,18 @@ def _ranks_from_score_columns(
     # the filtered rank is the number of unknown elements scored higher, where an element is
     # known if the edge (s, r, n) (for modified-object) or (n, r, o) (for modified-subject)
     # exists in known_edges_graph.
-
-    # FIXME(#870): this would be better without external IDs <-> ilocs translation
-    unmodified_nodes = known_edges_graph._nodes.ids.from_iloc(unmodified_node_ilocs)
-    true_rels = known_edges_graph._edges.types.from_iloc(true_rel_ilocs)
     if modified_object:
         neigh_func = known_edges_graph.out_nodes
     else:
         neigh_func = known_edges_graph.in_nodes
 
-    # collect all the neighbours into a single array to do one node_ids_to_ilocs call,
-    # which has relatively high constant cost
-    neighbours = []
-    columns = []
-    for batch_column, (unmodified, r) in enumerate(zip(unmodified_nodes, true_rels)):
-        this_neighs = neigh_func(unmodified, edge_types=[r])
-        neighbours.extend(this_neighs)
-        columns.extend(batch_column for _ in this_neighs)
+    for batch_column, (unmodified, r) in enumerate(
+        zip(unmodified_node_ilocs, true_rel_ilocs)
+    ):
+        this_neighs = neigh_func(unmodified, edge_types=[r], use_ilocs=True)
+        greater[this_neighs, batch_column] = False
+        greater_equal[this_neighs, batch_column] = False
 
-    neighbour_ilocs = known_edges_graph.node_ids_to_ilocs(neighbours)
-    greater[neighbour_ilocs, columns] = False
-    greater_equal[neighbour_ilocs, columns] = False
     # the actual elements should be counted as equal, whether or not it was a known edge or not
     greater_equal[true_modified_node_ilocs, range(batch_size)] = True
 
