@@ -29,12 +29,13 @@ from tensorflow.keras.utils import Sequence
 from scipy import sparse
 from ..core.graph import StellarGraph
 from ..core.utils import is_real_iterable, normalize_adj
+from ..connector.neo4j.graph import Neo4jStellarGraph
 from .base import Generator
 
 
 class ClusterNodeGenerator(Generator):
     """
-    A data generator for use with ClusterGCN models on homogeneous graphs, [1].
+    A data generator for use with ClusterGCN models on homogeneous graphs, see [1].
 
     The supplied graph G should be a StellarGraph object with node features.
     Use the :meth:`flow` method supplying the nodes and (optionally) targets
@@ -49,18 +50,22 @@ class ClusterNodeGenerator(Generator):
 
     Args:
         G (StellarGraph): a machine-learning StellarGraph-type graph
-        clusters (int or list): If int then it indicates the number of clusters (default is 1 that is the given graph).
-            If clusters is greater than 1, then nodes are uniformly at random assigned to a cluster. If list,
-            then it should be a list of lists of node IDs such that each list corresponds to a cluster of nodes
-            in G. The clusters should be non-overlapping.
-        q (float): The number of clusters to combine for each mini-batch. The default is 1.
-        lam (float): The mixture coefficient for adjacency matrix normalisation.
-        name (str): an optional name of the generator
+        clusters (int or list, optional): If int, it indicates the number of clusters (default is 1, corresponding to the entire graph).
+            If `clusters` is greater than 1, then nodes are randomly assigned to a cluster.
+            If list, then it should be a list of lists of node IDs, such that each list corresponds to a cluster of nodes
+            in `G`. The clusters should be non-overlapping.
+        q (int, optional): The number of clusters to combine for each mini-batch (default is 1).
+            The total number of clusters must be divisible by `q`.
+        lam (float, optional): The mixture coefficient for adjacency matrix normalisation (default is 0.1).
+            Valid values are in the interval [0, 1].
+        weighted (bool, optional): if True, use the edge weights from ``G``; if False, treat the
+            graph as unweighted.
+        name (str, optional): Name for the node generator.
     """
 
-    def __init__(self, G, clusters=1, q=1, lam=0.1, name=None):
+    def __init__(self, G, clusters=1, q=1, lam=0.1, weighted=False, name=None):
 
-        if not isinstance(G, StellarGraph):
+        if not isinstance(G, (StellarGraph, Neo4jStellarGraph)):
             raise TypeError("Graph must be a StellarGraph or StellarDiGraph object.")
 
         self.graph = G
@@ -68,6 +73,10 @@ class ClusterNodeGenerator(Generator):
         self.q = q  # The number of clusters to sample per mini-batch
         self.lam = lam
         self.clusters = clusters
+        self.method = "cluster_gcn"
+        self.multiplicity = 1
+        self.use_sparse = False
+        self.weighted = weighted
 
         if isinstance(clusters, list):
             self.k = len(clusters)
@@ -108,16 +117,14 @@ class ClusterNodeGenerator(Generator):
                 )
             )
 
-        # Check if the graph has features
-        G.check_graph_for_ml()
-
         self.node_list = list(G.nodes())
 
+        # if graph is a StellarGraph check that the graph has features
+        G.check_graph_for_ml(expensive_check=False)
         # Check that there is only a single node type
         _ = G.unique_node_type(
             "G: expected a graph with a single node type, found a graph with node types: %(found)s"
         )
-
         if isinstance(clusters, int):
             # We are not given graph clusters.
             # We are going to split the graph into self.k random clusters
@@ -138,8 +145,10 @@ class ClusterNodeGenerator(Generator):
         for i, c in enumerate(self.clusters):
             print(f"{i} cluster has size {len(c)}")
 
-        # Get the features for the nodes
-        self.features = G.node_features(self.node_list)
+        # Store the features of one node to allow graph ML models to peak at the feature dimension
+        # FIXME 1621: store feature_dimension here instead of features. This must also update ClusterGCN, and all
+        # fullbactch methods and generators
+        self.features = G.node_features(self.node_list[:1])
 
     def num_batch_dims(self):
         return 2
@@ -157,8 +166,8 @@ class ClusterNodeGenerator(Generator):
             name (str, optional): An optional name for the returned generator object.
 
         Returns:
-            A ClusterNodeSequence object to use with ClusterGCN in Keras
-            methods :meth:`fit`, :meth:`evaluate`, and :meth:`predict`
+            A :class:`ClusterNodeSequence` object to use with :class:`ClusterGCN` in Keras
+            methods :meth:`fit`, :meth:`evaluate`, and :meth:`predict`.
 
         """
         if targets is not None:
@@ -185,6 +194,7 @@ class ClusterNodeGenerator(Generator):
             node_ids=node_ids,
             q=self.q,
             lam=self.lam,
+            weighted=self.weighted,
             name=name,
         )
 
@@ -194,9 +204,9 @@ class ClusterNodeSequence(Sequence):
     A Keras-compatible data generator for node inference using ClusterGCN model.
     Use this class with the Keras methods :meth:`keras.Model.fit`,
         :meth:`keras.Model.evaluate`, and
-        :meth:`keras.Model.predict`,
+        :meth:`keras.Model.predict`.
 
-    This class should be created using the `.flow(...)` method of
+    This class should be created using the :meth:`flow` method of
     :class:`ClusterNodeGenerator`.
 
     Args:
@@ -224,6 +234,7 @@ class ClusterNodeSequence(Sequence):
         normalize_adj=True,
         q=1,
         lam=0.1,
+        weighted=False,
         name=None,
     ):
 
@@ -235,6 +246,7 @@ class ClusterNodeSequence(Sequence):
         self.normalize_adj = normalize_adj
         self.q = q
         self.lam = lam
+        self.weighted = weighted
         self.node_order = list()
         self._node_order_in_progress = list()
         self.__node_buffer = dict()
@@ -302,7 +314,7 @@ class ClusterNodeSequence(Sequence):
         # The next batch should be the adjacency matrix for the cluster and the corresponding feature vectors
         # and targets if available.
         cluster = self.clusters[index]
-        adj_cluster = self.graph.to_adjacency_matrix(cluster)
+        adj_cluster = self.graph.to_adjacency_matrix(cluster, weighted=self.weighted)
 
         if self.normalize_adj:
             adj_cluster = self._diagonal_enhanced_normalization(adj_cluster)
@@ -336,7 +348,8 @@ class ClusterNodeSequence(Sequence):
             # Dictionary to store node indices for quicker node index lookups
             # The list of indices of the target nodes in self.node_list
             cluster_target_indices = np.array(
-                [self.target_node_lookup[n] for n in target_nodes_in_cluster]
+                [self.target_node_lookup[n] for n in target_nodes_in_cluster],
+                dtype=np.int64,
             )
             cluster_targets = self.targets[cluster_target_indices]
             cluster_targets = cluster_targets.reshape((1,) + cluster_targets.shape)
