@@ -20,8 +20,9 @@ import pandas as pd
 import pytest
 import random
 from stellargraph.core.graph import *
+from stellargraph.core.indexed_array import IndexedArray
 from stellargraph.core.experimental import ExperimentalWarning
-from ..test_utils.alloc import snapshot, allocation_benchmark
+from ..test_utils.alloc import snapshot, peak, allocation_benchmark
 from ..test_utils.graphs import (
     example_graph_nx,
     example_graph,
@@ -30,6 +31,7 @@ from ..test_utils.graphs import (
     line_graph,
     weighted_hin,
     example_graph_random,
+    knowledge_graph,
 )
 
 from .. import test_utils
@@ -53,20 +55,27 @@ def create_graph_1(is_directed=False):
 
 
 def example_benchmark_graph(
-    feature_size=None, n_nodes=100, n_edges=200, n_types=4, features_in_nodes=True
+    feature_size=None,
+    n_nodes=100,
+    n_edges=200,
+    n_types=4,
+    features_in_nodes=True,
+    pandas_node_data=True,
 ):
     node_ids = np.arange(n_nodes)
     edges = pd.DataFrame(
         np.random.randint(0, n_nodes, size=(n_edges, 2)), columns=["source", "target"]
     )
 
-    if feature_size is None:
-        features = []
-    else:
-        features = np.ones((n_nodes, feature_size))
+    features = np.ones((n_nodes, 0 if feature_size is None else feature_size))
 
-    all_nodes = pd.DataFrame(features, index=node_ids)
-    nodes = {ty: all_nodes[node_ids % n_types == ty] for ty in range(n_types)}
+    feature_cls = pd.DataFrame if pandas_node_data else IndexedArray
+
+    def select_ty(ty):
+        idx = node_ids % n_types == ty
+        return feature_cls(features[idx, :], index=node_ids[idx])
+
+    nodes = {ty: select_ty(ty) for ty in range(n_types)}
 
     return nodes, edges
 
@@ -173,13 +182,13 @@ def test_graph_constructor_nodes_from_edges():
         StellarGraph(edges=1)
 
     with pytest.raises(
-        TypeError, match="edges.*: expected pandas DataFrame, found int"
+        TypeError, match="edges.*: expected IndexedArray or pandas DataFrame, found int"
     ):
         StellarGraph(edges={"a": 1})
 
     with pytest.raises(
         ValueError,
-        match=r"edges.*: expected 'source', 'target', 'weight' columns, found: 'weight'",
+        match=r"edges.*: expected 'source', 'target', 'weight' columns, found: 'x'",
     ):
         StellarGraph(edges=pd.DataFrame(columns=["x"]))
 
@@ -197,6 +206,118 @@ def test_graph_constructor_edge_labels():
     ]
 
 
+def test_graph_constructor_internal():
+    orig = example_graph_random(node_types=3, edge_types=3, is_directed=True)
+    undir_g = StellarGraph(orig._nodes, orig._edges)
+    dir_g = StellarDiGraph(orig._nodes, orig._edges)
+
+    assert not undir_g.is_directed()
+    assert dir_g.is_directed()
+    for g in [undir_g, dir_g]:
+        assert g.node_types == orig.node_types
+        for t in orig.node_types:
+            np.testing.assert_array_equal(
+                g.node_features(node_type=t), orig.node_features(node_type=t)
+            )
+        assert g.edges(include_edge_type=True) == orig.edges(include_edge_type=True)
+
+    with pytest.raises(
+        TypeError, match="edges: expected type 'EdgeData' .* found dict"
+    ):
+        StellarGraph(orig._nodes, {})
+
+    with pytest.raises(
+        TypeError, match="nodes: expected type 'NodeData' .* found DataFrame"
+    ):
+        StellarGraph(pd.DataFrame(index=[0]), orig._edges)
+
+    # check that each parameter is validated as being the default. This explicitly lists the
+    # parameters, to not rely on `__kwdefaults__` since the internal implementation uses that too
+    # (at the time of writing).
+    non_default = [
+        "source_column",
+        "target_column",
+        "edge_weight_column",
+        "node_type_default",
+        "edge_type_default",
+        "edge_type_column",
+        "dtype",
+    ]
+    unchecked = {
+        # is_directed is allowed to be specified
+        "is_directed",
+        # don't check the legacy forms
+        "graph",
+        "node_type_name",
+        "edge_type_name",
+        "node_features",
+    }
+    # check that we seem to be covering all the relevant parameters. Also, if something fundamental
+    # changes with the class this will hopefully catch the testing being invalidated.
+    assert set(non_default) == set(StellarGraph.__init__.__kwdefaults__) - unchecked
+
+    for param in non_default:
+        with pytest.raises(
+            ValueError, match=f"{param}: expected the default value .* found <object"
+        ):
+            # specify a junk object
+            StellarGraph(orig._nodes, orig._edges, **{param: object()})
+
+
+@pytest.fixture(params=["IndexedArray", "NumPy"])
+def rowframe_convert(request):
+    return IndexedArray if request.param == "IndexedArray" else (lambda arr: arr)
+
+
+def test_graph_constructor_rowframe_numpy_homogeneous(rowframe_convert):
+    empty = np.empty((0, 0))
+    g = StellarGraph(rowframe_convert(empty))
+    np.testing.assert_array_equal(g.nodes(), [])
+    assert g.node_features() is empty
+
+    arr = np.random.rand(3, 4, 5)
+    edges = pd.DataFrame({"source": [0, 1], "target": [2, 2]})
+    g = StellarGraph(rowframe_convert(arr), edges)
+    np.testing.assert_array_equal(g.nodes(), [0, 1, 2])
+    assert g.node_features() is arr
+
+
+def test_graph_constructor_rowframe_numpy_heterogeneous(rowframe_convert):
+    arr1 = np.random.rand(3, 4, 5)
+    arr2 = np.random.rand(6, 7)
+    frame2 = IndexedArray(arr2, index=range(100, 106))
+
+    g = StellarGraph({"a": rowframe_convert(arr1), "b": frame2})
+    np.testing.assert_array_equal(g.nodes(), [0, 1, 2, 100, 101, 102, 103, 104, 105])
+    assert g.node_features(node_type="a") is arr1
+    assert g.node_features(node_type="b") is arr2
+
+
+def test_graph_constructor_rowframe_numpy_invalid():
+    arr1 = np.random.rand(3, 4, 5)
+    arr2 = np.random.rand(6, 7)
+
+    with pytest.raises(ValueError, match="expected IDs .*, found .* more: 0, 1, 2"):
+        # using an array directly twice will cause the auto-range-IDs to overlap
+        StellarGraph({"a": arr1, "b": arr2})
+
+    with pytest.raises(ValueError, match="expected IDs .*, found .* more: 1"):
+        StellarGraph(IndexedArray(index=[1, 1]))
+
+    with pytest.raises(
+        ValueError, match="edges: expected all source .* found some missing: 'a'"
+    ):
+        StellarGraph(arr1, pd.DataFrame({"source": ["a"], "target": ["b"]}))
+
+    with pytest.raises(
+        ValueError, match="edges: expected all source .* found some missing: 'b'"
+    ):
+        StellarGraph(
+            IndexedArray(index=["a", "c"]),
+            pd.DataFrame({"source": ["a"], "target": ["b"]}),
+        )
+
+
 def test_info():
     sg = create_graph_1()
     info_str = sg.info()
@@ -295,6 +416,50 @@ def test_node_ilocs_to_ids():
     assert (node_ids == expected_node_ids).all()
 
 
+def test_node_type_names_to_from_ilocs():
+    sg = example_hin_1()
+
+    def both_ways(names, ilocs):
+        np.testing.assert_array_equal(sg.node_type_names_to_ilocs(names), ilocs)
+        np.testing.assert_array_equal(sg.node_type_ilocs_to_names(ilocs), names)
+
+    both_ways([], [])
+    both_ways(["A"], [0])
+    both_ways(["B", "A", "A", "B"], [1, 0, 0, 1])
+
+    with pytest.raises(KeyError, match="'C'.*0"):
+        sg.node_type_names_to_ilocs(["C", "A", 0])
+
+    with pytest.raises(IndexError, match="index 100 .* size 2"):
+        sg.node_type_ilocs_to_names([100])
+
+    with pytest.raises(IndexError, match="index -100 .* size 2"):
+        sg.node_type_ilocs_to_names([-100])
+
+
+def test_edge_type_names_to_from_ilocs(knowledge_graph):
+    def both_ways(names, ilocs):
+        np.testing.assert_array_equal(
+            knowledge_graph.edge_type_names_to_ilocs(names), ilocs
+        )
+        np.testing.assert_array_equal(
+            knowledge_graph.edge_type_ilocs_to_names(ilocs), names
+        )
+
+    both_ways([], [])
+    both_ways(["W"], [0])
+    both_ways(["Z", "X", "W", "W", "Z", "Z"], [3, 1, 0, 0, 3, 3])
+
+    with pytest.raises(KeyError, match="'U'.*0"):
+        knowledge_graph.edge_type_names_to_ilocs(["U", "W", 0])
+
+    with pytest.raises(IndexError, match="index 100 .* size 4"):
+        knowledge_graph.edge_type_ilocs_to_names([100])
+
+    with pytest.raises(IndexError, match="index -100 .* size 4"):
+        knowledge_graph.edge_type_ilocs_to_names([-100])
+
+
 def test_feature_conversion_from_nodes():
     sg = example_graph(feature_size=8)
     aa = sg.node_features([1, 2, 3, 4])
@@ -304,6 +469,40 @@ def test_feature_conversion_from_nodes():
     assert sg.node_feature_sizes()["default"] == 8
 
 
+def test_node_feature_sizes_shapes():
+    g = example_hin_1(feature_sizes={"A": 4, "B": (6, 7)})
+
+    assert g.node_feature_shapes() == {"A": (4,), "B": (6, 7)}
+    with pytest.raises(
+        ValueError,
+        match=r"node_feature_sizes expects node types .* found type 'B' with feature shape \(6, 7\)",
+    ):
+        g.node_feature_sizes()
+
+    assert g.node_feature_shapes(node_types=["A"]) == {"A": (4,)}
+    assert g.node_feature_sizes(node_types=["A"]) == {"A": 4}
+
+    assert g.node_feature_shapes(node_types=["B"]) == {"B": (6, 7)}
+    with pytest.raises(
+        ValueError,
+        match=r"node_feature_sizes expects node types .* found type 'B' with feature shape \(6, 7\)",
+    ):
+        g.node_feature_sizes(node_types=["B"])
+
+
+def test_edge_feature_sizes_shapes():
+    # edges don't support multidimensional features (yet...)
+    g = example_hin_1(feature_sizes={"F": 4, "R": 6}, edge_features=True)
+
+    assert g.edge_feature_shapes() == {"F": (4,), "R": (6,)}
+
+    assert g.edge_feature_shapes(edge_types=["F"]) == {"F": (4,)}
+    assert g.edge_feature_sizes(edge_types=["F"]) == {"F": 4}
+
+    assert g.edge_feature_shapes(edge_types=["R"]) == {"R": (6,)}
+    assert g.edge_feature_sizes(edge_types=["R"]) == {"R": 6}
+
+
 def test_node_features():
     feature_sizes = {"A": 4, "B": 6}
     sg = example_hin_1(feature_sizes=feature_sizes)
@@ -395,6 +594,98 @@ def test_null_node_feature():
         sg.node_features([None, None])
 
 
+def test_edge_features():
+    feature_sizes = {"F": 4, "R": 6}
+    sg = example_hin_1(feature_sizes=feature_sizes, edge_features=True)
+
+    one_zero = np.array([[-1] * 6, [0] * 6])
+
+    # inference
+    np.testing.assert_array_equal(sg.edge_features(edges=[1, 0]), one_zero)
+    # specified
+    np.testing.assert_array_equal(
+        sg.edge_features(edges=[1, 0], edge_type="R"), one_zero
+    )
+
+    # wrong type
+    with pytest.raises(ValueError, match="unknown IDs"):
+        sg.edge_features(edges=[1, 0], edge_type="F")
+
+    # mixed types, inference
+    with pytest.raises(ValueError, match="all edges must have the same type"):
+        sg.edge_features(edges=[0, 100])
+
+    # mixed types, specified
+    with pytest.raises(ValueError, match="unknown IDs"):
+        sg.edge_features(edges=[0, 100], edge_type="R")
+
+
+def test_edge_features_edge_type():
+    feature_sizes = {"F": 4, "R": 6}
+    sg = example_hin_1(feature_sizes=feature_sizes, edge_features=True)
+
+    np.testing.assert_array_equal(
+        sg.edge_features(edge_type="R"),
+        [[0] * 6, [-1] * 6, [-2] * 6, [-3] * 6, [-4] * 6],
+    )
+    np.testing.assert_array_equal(sg.edge_features(edge_type="F"), [[-100] * 4])
+
+
+def test_edge_features_edge_type_inference():
+    one_type = example_graph(edge_feature_size=4)
+    np.testing.assert_array_equal(
+        one_type.edge_features(), [[0] * 4, [-1] * 4, [-2] * 4, [-3] * 4]
+    )
+
+    # short-cut inference of the edge type for a subset of the edges with a homogenous graph
+    np.testing.assert_array_equal(one_type.edge_features([1]), [[-1] * 4])
+
+    # inference doesn't work for a heterogeneous graph:
+    feature_sizes = {"F": 4, "R": 6}
+    many_types = example_hin_1(feature_sizes=feature_sizes, edge_features=True)
+
+    with pytest.raises(
+        ValueError,
+        match="edge_type: in a non-homogeneous graph, expected a edge type and/or 'edges' to be passed; found neither 'edge_type' nor 'edges', and the graph has edge types: 'F', 'R'",
+    ):
+        many_types.edge_features()
+
+
+def test_edge_features_missing_id():
+    sg = example_graph(edge_feature_size=6)
+    with pytest.raises(KeyError, match=r"\[1000, 2000\]"):
+        sg.edge_features([1, 1000, None, 2000])
+
+
+def test_null_edge_feature():
+    sg = example_graph(edge_feature_size=6)
+    aa = sg.edge_features([1, None, 2, None])
+    assert aa.shape == (4, 6)
+    assert aa[:, 0] == pytest.approx([-1, 0, -2, 0])
+
+    sg = example_hin_1(
+        feature_sizes={"F": 4, "R": 2}, reverse_order=True, edge_features=True
+    )
+
+    # Test feature for null edge, without edge type
+    ab = sg.edge_features([None, 4, None])
+    assert ab.shape == (3, 2)
+    assert ab[:, 0] == pytest.approx([0, -4, 0])
+
+    # Test feature for null edge, edge type
+    ab = sg.edge_features([None, 4, None], "R")
+    assert ab.shape == (3, 2)
+    assert ab[:, 0] == pytest.approx([0, -4, 0])
+
+    # Test feature for null edge, wrong type
+    with pytest.raises(ValueError):
+        sg.edge_features([None, 4, None], "F")
+
+    # Test null-edge with no type
+    with pytest.raises(ValueError):
+        sg.edge_features([None, None])
+
+
 def test_node_types():
     sg = example_graph(feature_size=6)
     assert sg.node_types == {"default"}
@@ -548,7 +839,12 @@ def test_edges_include_edge_type(use_ilocs):
 
     expected = r | f
     if use_ilocs:
-        expected = {tuple(g.node_ids_to_ilocs(x[:2])) + (x[2],) for x in expected}
+        expected = {
+            tuple(g.node_ids_to_ilocs(x[:2]))
+            + tuple(g.edge_type_names_to_ilocs([x[2]]))
+            for x in expected
+        }
+
     expected = normalize_edges(expected, directed=False)
     assert (
         normalize_edges(
@@ -702,7 +998,7 @@ def test_benchmark_get_neighbours(benchmark, use_ilocs):
     # get the neigbours of every node in the graph
     def f():
         for i in range(num_nodes):
-            sg.neighbors(i, use_ilocs=use_ilocs)
+            sg.neighbor_arrays(i, use_ilocs=use_ilocs)
 
     benchmark(f)
 
@@ -755,62 +1051,97 @@ def test_benchmark_get_features(
     benchmark(f)
 
 
+def _run_creation_benchmark(
+    benchmarker, input_data, feature_size, num_nodes, num_edges
+):
+    nodes, edges = example_benchmark_graph(
+        feature_size=feature_size,
+        n_nodes=num_nodes,
+        n_edges=num_edges,
+        pandas_node_data=input_data == "pandas",
+    )
+
+    def f():
+        return StellarGraph(nodes=nodes, edges=edges)
+
+    benchmarker(f)
+
+
 @pytest.mark.benchmark(group="StellarGraph creation (time)")
+@pytest.mark.parametrize("input_data", ["pandas", "rowframe"])
 # various element counts, to give an indication of the relationship
 # between those and memory use (0,0 gives the overhead of the
 # StellarGraph object itself, without any data)
-@pytest.mark.parametrize("num_nodes,num_edges", [(0, 0), (100, 200), (1000, 5000)])
+@pytest.mark.parametrize("num_nodes,num_edges", [(0, 0), (1000, 5000), (20000, 100000)])
 # features or not, to capture their cost
 @pytest.mark.parametrize("feature_size", [None, 100])
-@pytest.mark.parametrize("force_adj_lists", [None, "directed", "undirected", "both"])
-def test_benchmark_creation(
-    benchmark, feature_size, num_nodes, num_edges, force_adj_lists
-):
-    nodes, edges = example_benchmark_graph(
-        feature_size, num_nodes, num_edges, features_in_nodes=True
-    )
-
-    def f():
-        sg = StellarGraph(nodes=nodes, edges=edges)
-        if force_adj_lists == "directed":
-            sg._edges._init_directed_adj_lists()
-        elif force_adj_lists == "undirected":
-            sg._edges._init_undirected_adj_lists()
-        elif force_adj_lists == "both":
-            sg._edges._init_undirected_adj_lists()
-            sg._edges._init_directed_adj_lists()
-        return sg
-
-    benchmark(f)
+def test_benchmark_creation(benchmark, input_data, feature_size, num_nodes, num_edges):
+    _run_creation_benchmark(benchmark, input_data, feature_size, num_nodes, num_edges)
 
 
-@pytest.mark.benchmark(group="StellarGraph creation", timer=snapshot)
-# various element counts, to give an indication of the relationship
-# between those and memory use (0,0 gives the overhead of the
-# StellarGraph object itself, without any data)
+@pytest.mark.benchmark(group="StellarGraph creation (size)", timer=snapshot)
+@pytest.mark.parametrize("input_data", ["pandas", "rowframe"])
 @pytest.mark.parametrize("num_nodes,num_edges", [(0, 0), (100, 200), (1000, 5000)])
-# features or not, to capture their cost
 @pytest.mark.parametrize("feature_size", [None, 100])
-@pytest.mark.parametrize("force_adj_lists", [None, "directed", "undirected", "both"])
 def test_allocation_benchmark_creation(
-    allocation_benchmark, feature_size, num_nodes, num_edges, force_adj_lists
+    allocation_benchmark, input_data, feature_size, num_nodes, num_edges
 ):
-    nodes, edges = example_benchmark_graph(
-        feature_size, num_nodes, num_edges, features_in_nodes=True
+    _run_creation_benchmark(
+        allocation_benchmark, input_data, feature_size, num_nodes, num_edges
     )
 
-    def f():
-        sg = StellarGraph(nodes=nodes, edges=edges)
-        if force_adj_lists == "directed":
-            sg._edges._init_directed_adj_lists()
-        elif force_adj_lists == "undirected":
-            sg._edges._init_undirected_adj_lists()
-        elif force_adj_lists == "both":
-            sg._edges._init_undirected_adj_lists()
-            sg._edges._init_directed_adj_lists()
-        return sg
 
-    allocation_benchmark(f)
+@pytest.mark.benchmark(group="StellarGraph creation (peak)", timer=peak)
+@pytest.mark.parametrize("input_data", ["pandas", "rowframe"])
+@pytest.mark.parametrize("num_nodes,num_edges", [(0, 0), (100, 200), (1000, 5000)])
+@pytest.mark.parametrize("feature_size", [None, 100])
+def test_allocation_benchmark_creation_peak(
+    allocation_benchmark, input_data, feature_size, num_nodes, num_edges
+):
+    _run_creation_benchmark(
+        allocation_benchmark, input_data, feature_size, num_nodes, num_edges
+    )
+
+
+def _run_adj_list_benchmark(benchmarker, num_nodes, num_edges, force_adj_lists):
+    nodes, edges = example_benchmark_graph(n_nodes=num_nodes, n_edges=num_edges)
+
+    sg = StellarGraph(nodes=nodes, edges=edges)
+
+    def f():
+        if force_adj_lists == "directed":
+            return sg._edges._create_undirected_adj_lists()
+        elif force_adj_lists == "undirected":
+            return sg._edges._create_directed_adj_lists()
+
+    benchmarker(f)
+
+
+@pytest.mark.benchmark(group="StellarGraph adjacency lists (time)")
+@pytest.mark.parametrize(
+    "num_nodes,num_edges", [(100, 200), (1000, 5000), (20000, 100000)]
+)
+@pytest.mark.parametrize("force_adj_lists", ["directed", "undirected"])
+def test_benchmark_adj_list(benchmark, num_nodes, num_edges, force_adj_lists):
+    _run_adj_list_benchmark(benchmark, num_nodes, num_edges, force_adj_lists)
+
+
+@pytest.mark.benchmark(group="StellarGraph adjacency lists (size)", timer=snapshot)
+@pytest.mark.parametrize("num_nodes,num_edges", [(100, 200), (1000, 5000)])
+@pytest.mark.parametrize("force_adj_lists", ["directed", "undirected"])
+def test_allocation_benchmark_adj_list(
+    allocation_benchmark, num_nodes, num_edges, force_adj_lists
+):
+    _run_adj_list_benchmark(allocation_benchmark, num_nodes, num_edges, force_adj_lists)
+
+
+@pytest.mark.benchmark(group="StellarGraph adjacency lists (peak)", timer=peak)
+@pytest.mark.parametrize("num_nodes,num_edges", [(100, 200), (1000, 5000)])
+@pytest.mark.parametrize("force_adj_lists", ["directed", "undirected"])
+def test_allocation_benchmark_adj_list_peak(
+    allocation_benchmark, num_nodes, num_edges, force_adj_lists
+):
+    _run_adj_list_benchmark(allocation_benchmark, num_nodes, num_edges, force_adj_lists)
 
 
 def example_weighted_hin(is_directed=True):
@@ -836,6 +1167,13 @@ def example_unweighted_hom(is_directed=True):
     return StellarDiGraph(nodes, edges) if is_directed else StellarGraph(nodes, edges)
 
 
+def _edge_types_or_ilocs(graph, use_ilocs, names):
+    if use_ilocs:
+        return graph.edge_type_names_to_ilocs(names)
+
+    return names
+
+
 @pytest.mark.parametrize("use_ilocs", [True, False])
 @pytest.mark.parametrize("is_directed", [True, False])
 def test_neighbors_weighted_hin(is_directed, use_ilocs):
@@ -853,11 +1191,12 @@ def test_neighbors_weighted_hin(is_directed, use_ilocs):
         zip(expected_nodes, expected_weights),
     )
 
+    edge_types = _edge_types_or_ilocs(graph, use_ilocs, ["AB"])
     expected_nodes = graph.node_ids_to_ilocs([2, 3]) if use_ilocs else [2, 3]
     expected_weights = [10.0, 10.0]
     assert_items_equal(
         graph.neighbors(
-            node, include_edge_weight=True, edge_types=["AB"], use_ilocs=use_ilocs
+            node, include_edge_weight=True, edge_types=edge_types, use_ilocs=use_ilocs
         ),
         zip(expected_nodes, expected_weights),
     )
@@ -882,9 +1221,10 @@ def test_neighbors_unweighted_hom(is_directed, use_ilocs):
         graph.neighbors(node, include_edge_weight=True, use_ilocs=use_ilocs),
         zip(expected_nodes, expected_weights),
     )
+
     assert_items_equal(
         graph.neighbors(
-            node, include_edge_weight=True, edge_types=["AB"], use_ilocs=use_ilocs
+            node, include_edge_weight=True, edge_types=[10], use_ilocs=use_ilocs
         ),
         [],
     )
@@ -910,6 +1250,7 @@ def test_in_nodes_weighted_hin(use_ilocs):
     node = graph.node_ids_to_ilocs([1])[0] if use_ilocs else 1
     expected_nodes = graph.node_ids_to_ilocs([0, 0]) if use_ilocs else [0, 0]
     expected_weighted = zip(expected_nodes, [0.0, 1.0])
+    edge_types = _edge_types_or_ilocs(graph, use_ilocs, ["AB"])
 
     assert_items_equal(graph.in_nodes(node, use_ilocs=use_ilocs), expected_nodes)
     assert_items_equal(
@@ -918,7 +1259,7 @@ def test_in_nodes_weighted_hin(use_ilocs):
     )
     assert_items_equal(
         graph.in_nodes(
-            node, include_edge_weight=True, edge_types=["AB"], use_ilocs=use_ilocs
+            node, include_edge_weight=True, edge_types=edge_types, use_ilocs=use_ilocs
         ),
         [],
     )
@@ -938,7 +1279,7 @@ def test_in_nodes_unweighted_hom(use_ilocs):
     )
     assert_items_equal(
         graph.in_nodes(
-            node, include_edge_weight=True, edge_types=["AA"], use_ilocs=use_ilocs
+            node, include_edge_weight=True, edge_types=[10], use_ilocs=use_ilocs
         ),
         [],
     )
@@ -958,7 +1299,7 @@ def test_out_nodes_weighted_hin(use_ilocs):
     )
     assert_items_equal(
         graph.out_nodes(
-            node, include_edge_weight=True, edge_types=["AA"], use_ilocs=use_ilocs
+            node, include_edge_weight=True, edge_types=[10], use_ilocs=use_ilocs
         ),
         [],
     )
@@ -978,7 +1319,7 @@ def test_out_nodes_unweighted_hom(use_ilocs):
     )
     assert_items_equal(
         graph.out_nodes(
-            node, include_edge_weight=True, edge_types=["AB"], use_ilocs=use_ilocs
+            node, include_edge_weight=True, edge_types=[10], use_ilocs=use_ilocs
         ),
         [],
     )
@@ -1001,7 +1342,11 @@ def test_isolated_node_neighbor_methods(is_directed, use_ilocs):
 def test_info_homogeneous(is_directed):
 
     g = example_graph(
-        feature_size=12, node_label="ABC", edge_label="xyz", is_directed=is_directed
+        feature_size=12,
+        node_label="ABC",
+        edge_label="xyz",
+        is_directed=is_directed,
+        edge_feature_size=34,
     )
 
     if is_directed:
@@ -1023,12 +1368,15 @@ def test_info_homogeneous(is_directed):
 
  Edge types:
     ABC-xyz->ABC: [4]
-        Weights: all 1 (default)"""
+        Weights: all 1 (default)
+        Features: float32 vector, length 34"""
     )
 
 
 def test_info_heterogeneous():
-    g = example_hin_1({"A": 0, "B": 34}, reverse_order=True)
+    g = example_hin_1(
+        {"A": 0, "B": (34, 4), "F": 0, "R": 56}, reverse_order=True, edge_features=True
+    )
     # literal match to check the output is good for human consumption
     assert (
         g.info()
@@ -1041,16 +1389,16 @@ StellarGraph: Undirected multigraph
     Features: none
     Edge types: A-R->B
   B: [3]
-    Features: float32 vector, length 34
+    Features: float32 tensor, shape (34, 4)
     Edge types: B-F->B, B-R->A
 
  Edge types:
-    A-R->B: [3]
-        Weights: all 1 (default)
-    B-R->A: [2]
+    A-R->B: [5]
         Weights: all 1 (default)
+        Features: float32 vector, length 56
     B-F->B: [1]
-        Weights: all 10"""
+        Weights: all 10
+        Features: none"""
     )
 
 
@@ -1071,18 +1419,21 @@ StellarGraph: Undirected multigraph
     Edge types: B-T->A, B-U->A
 
  Edge types:
+    A-U->B: [3]
+        Weights: range=[4, 5], mean=4.66667, std=0.57735
+        Features: none
     A-T->B: [3]
         Weights: all 2
-    A-U->B: [2]
-        Weights: range=[4, 5], mean=4.5, std=0.707107
+        Features: none
     A-U->A: [2]
         Weights: range=[2, 3], mean=2.5, std=0.707107
+        Features: none
     A-S->A: [2]
         Weights: all 2
+        Features: none
     A-R->A: [2]
         Weights: all 1 (default)
-    B-U->A: [1]
-        Weights: all 5"""
+        Features: none"""
     )
 
 
@@ -1206,44 +1557,64 @@ StellarGraph: Undirected multigraph
  Edge types:
     n_21-e_22->n_21: [45]
         Weights: all 1 (default)
+        Features: none
     n_21-e_21->n_21: [43]
         Weights: all 1 (default)
+        Features: none
     n_21-e_20->n_21: [41]
         Weights: all 1 (default)
+        Features: none
     n_21-e_19->n_21: [39]
         Weights: all 1 (default)
+        Features: none
     n_21-e_18->n_21: [37]
         Weights: all 1 (default)
+        Features: none
     n_21-e_17->n_21: [35]
         Weights: all 1 (default)
+        Features: none
     n_21-e_16->n_21: [33]
         Weights: all 1 (default)
+        Features: none
     n_21-e_15->n_21: [31]
         Weights: all 1 (default)
+        Features: none
     n_21-e_14->n_21: [29]
         Weights: all 1 (default)
+        Features: none
     n_21-e_13->n_21: [27]
         Weights: all 1 (default)
+        Features: none
     n_21-e_12->n_21: [25]
         Weights: all 1 (default)
+        Features: none
     n_21-e_11->n_21: [23]
         Weights: all 1 (default)
+        Features: none
     n_21-e_10->n_21: [21]
         Weights: all 1 (default)
+        Features: none
     n_21-e_9->n_21: [19]
         Weights: all 1 (default)
+        Features: none
     n_21-e_8->n_21: [17]
         Weights: all 1 (default)
+        Features: none
     n_21-e_7->n_21: [15]
         Weights: all 1 (default)
+        Features: none
     n_21-e_6->n_21: [13]
         Weights: all 1 (default)
+        Features: none
     n_21-e_5->n_21: [11]
         Weights: all 1 (default)
+        Features: none
     n_21-e_4->n_21: [9]
         Weights: all 1 (default)
+        Features: none
     n_21-e_3->n_21: [7]
         Weights: all 1 (default)
+        Features: none
     ... (3 more)"""
     )
 
@@ -1265,8 +1636,10 @@ StellarGraph: Undirected multigraph
  Edge types:
     n_21-e_22->n_21: [45]
         Weights: all 1 (default)
+        Features: none
     n_21-e_21->n_21: [43]
         Weights: all 1 (default)
+        Features: none
     ... (21 more)"""
     )
 
@@ -1347,50 +1720,73 @@ StellarGraph: Undirected multigraph
  Edge types:
     n_21-e_22->n_21: [45]
         Weights: all 1 (default)
+        Features: none
     n_21-e_21->n_21: [43]
         Weights: all 1 (default)
+        Features: none
     n_21-e_20->n_21: [41]
         Weights: all 1 (default)
+        Features: none
     n_21-e_19->n_21: [39]
         Weights: all 1 (default)
+        Features: none
     n_21-e_18->n_21: [37]
         Weights: all 1 (default)
+        Features: none
     n_21-e_17->n_21: [35]
         Weights: all 1 (default)
+        Features: none
     n_21-e_16->n_21: [33]
         Weights: all 1 (default)
+        Features: none
     n_21-e_15->n_21: [31]
         Weights: all 1 (default)
+        Features: none
     n_21-e_14->n_21: [29]
         Weights: all 1 (default)
+        Features: none
     n_21-e_13->n_21: [27]
         Weights: all 1 (default)
+        Features: none
     n_21-e_12->n_21: [25]
         Weights: all 1 (default)
+        Features: none
     n_21-e_11->n_21: [23]
         Weights: all 1 (default)
+        Features: none
     n_21-e_10->n_21: [21]
         Weights: all 1 (default)
+        Features: none
     n_21-e_9->n_21: [19]
         Weights: all 1 (default)
+        Features: none
     n_21-e_8->n_21: [17]
         Weights: all 1 (default)
+        Features: none
     n_21-e_7->n_21: [15]
         Weights: all 1 (default)
+        Features: none
     n_21-e_6->n_21: [13]
         Weights: all 1 (default)
+        Features: none
     n_21-e_5->n_21: [11]
         Weights: all 1 (default)
+        Features: none
     n_21-e_4->n_21: [9]
         Weights: all 1 (default)
+        Features: none
     n_21-e_3->n_21: [7]
         Weights: all 1 (default)
+        Features: none
     n_21-e_2->n_21: [5]
         Weights: all 1 (default)
+        Features: none
     n_21-e_1->n_21: [3]
         Weights: all 1 (default)
+        Features: none
     n_21-e_0->n_21: [1]
-        Weights: all 1 (default)"""
+        Weights: all 1 (default)
+        Features: none"""
     )
 
 
@@ -1829,7 +2225,7 @@ def test_node_degrees(use_ilocs):
     degrees = g.node_degrees(use_ilocs=use_ilocs)
 
     # expected node degrees - keys are node ids
-    expected = {0: 1, 1: 2, 2: 1, 3: 1, 4: 4, 5: 3}
+    expected = {0: 1, 1: 2, 2: 1, 3: 1, 4: 4, 5: 3, 6: 0}
     if use_ilocs:
         for node_id in expected.keys():
             node_iloc = g.node_ids_to_ilocs([node_id])[0]
@@ -1858,3 +2254,49 @@ def test_unique_node_type():
         ValueError, match="^ABC custom message 'n-0', 'n-1', 'n-2', 'n-3' 123$"
     ):
         many_types.unique_node_type("ABC custom message %(found)s 123")
+
+
+def test_unique_edge_type():
+    one_type = example_graph_random(node_types=10, edge_types=1)
+
+    assert one_type.unique_edge_type() == "e-0"
+
+    many_types = example_graph_random(node_types=1, edge_types=4)
+
+    with pytest.raises(
+        ValueError,
+        match="Expected only one edge type for 'unique_edge_type', found: 'e-0', 'e-1', 'e-2', 'e-3'",
+    ):
+        many_types.unique_edge_type()
+
+    with pytest.raises(ValueError, match="^ABC custom message 123$"):
+        many_types.unique_edge_type("ABC custom message 123")
+
+    with pytest.raises(
+        ValueError, match="^ABC custom message 'e-0', 'e-1', 'e-2', 'e-3' 123$"
+    ):
+        many_types.unique_edge_type("ABC custom message %(found)s 123")
+
+
+def test_bulk_node_types():
+    sg = example_hin_1(feature_sizes={}, reverse_order=True)
+    node_ilocs = sg.nodes(use_ilocs=True)
+    node_types = sg.node_type(node_ilocs, use_ilocs=True)
+    assert (node_types[:4] == "A").all()
+    assert (node_types[4:] == "B").all()
+
+
+def test_correct_adjacency_list_type():
+
+    # the dtype of index into "sg._edges._edges_dict"
+    # can be larger than "sg._edges.ids.dtype"
+    # because there are 2 undirected edges for every (non-self loop) directed edge
+    # this test checks this edge case
+    cycle_graph = nx.cycle_graph(200)
+    sg = StellarGraph.from_networkx(cycle_graph)
+    sg._edges._init_undirected_adj_lists()
+
+    assert sg.number_of_edges() == 200
+    assert sg._edges.ids.dtype == np.uint8
+    assert all(deg == 2 for deg in sg.node_degrees().values())
+    assert sg._edges._edges_dict.flat.dtype == np.uint16
