@@ -15,8 +15,9 @@
 # limitations under the License.
 
 import pytest
+import numpy as np
 from stellargraph.data.explorer import UniformRandomWalk
-from ..test_utils.graphs import create_test_graph
+from ..test_utils.graphs import create_test_graph, example_graph_random
 
 
 class TestUniformRandomWalk(object):
@@ -41,13 +42,13 @@ class TestUniformRandomWalk(object):
             urw.run(nodes=nodes, n=0, length=length, seed=seed)
         with pytest.raises(ValueError):
             urw.run(nodes=nodes, n=-121, length=length, seed=seed)
-        with pytest.raises(ValueError):
+        with pytest.raises(TypeError):
             urw.run(nodes=nodes, n=21.4, length=length, seed=seed)
-        with pytest.raises(ValueError):
+        with pytest.raises(TypeError):
             urw.run(nodes=nodes, n=-0.5, length=length, seed=seed)
-        with pytest.raises(ValueError):
+        with pytest.raises(TypeError):
             urw.run(nodes=nodes, n=0.0001, length=length, seed=seed)
-        with pytest.raises(ValueError):
+        with pytest.raises(TypeError):
             urw.run(nodes=nodes, n="2", length=length, seed=seed)
 
         # length has to be positive integer
@@ -55,17 +56,17 @@ class TestUniformRandomWalk(object):
             urw.run(nodes=nodes, n=n, length=0, seed=seed)
         with pytest.raises(ValueError):
             urw.run(nodes=nodes, n=n, length=-5, seed=seed)
-        with pytest.raises(ValueError):
+        with pytest.raises(TypeError):
             urw.run(nodes=nodes, n=n, length=11.9, seed=seed)
-        with pytest.raises(ValueError):
+        with pytest.raises(TypeError):
             urw.run(nodes=nodes, n=n, length=-9.9, seed=seed)
-        with pytest.raises(ValueError):
+        with pytest.raises(TypeError):
             urw.run(nodes=nodes, n=n, length="10", seed=seed)
 
         # seed has to be None, 0,  or positive integer
         with pytest.raises(ValueError):
             urw.run(nodes=nodes, n=n, length=length, seed=-1)
-        with pytest.raises(ValueError):
+        with pytest.raises(TypeError):
             urw.run(nodes=nodes, n=n, length=length, seed=1010.8)
 
         # If no root nodes are given, an empty list is returned which is not an error but I thought this method
@@ -222,12 +223,27 @@ class TestUniformRandomWalk(object):
             for node in subgraph:
                 assert node == "self loner"  # all nodes should be the same node
 
-    def test_benchmark_uniformrandomwalk(self, benchmark):
-
+    def test_init_parameters(self):
         g = create_test_graph()
+
+        nodes = ["0", 2]
+        n = 1
+        length = 2
+        seed = 0
+
+        urw = UniformRandomWalk(g, n=n, length=length, seed=seed)
+        urw_no_params = UniformRandomWalk(g)
+
+        run_1 = urw.run(nodes=nodes)
+        run_2 = urw_no_params.run(nodes=nodes, n=n, length=length, seed=seed)
+        assert np.array_equal(run_1, run_2)
+
+    def test_benchmark_uniformrandomwalk(self, benchmark):
+        g = example_graph_random(n_nodes=100, n_edges=500)
         urw = UniformRandomWalk(g)
 
-        nodes = ["0"]  # this node has no edges including itself
+        nodes = np.arange(0, 50)
+        n = 2
         n = 5
         length = 5
 
